__all__ = ['EVENT_HANDLER_EVENT_TYPE_MAP']

EVENT_HANDLER_EVENT_TYPE_MAP: dict[str, str] = {
    'onabort': 'abort',
    'onauxclick': 'auxclick',
    'onbeforeinput': 'beforeinput',
    'onbeforematch': 'beforematch',
    'onbeforetoggle': 'beforetoggle',
    'oncancel': 'cancel',
    'oncanplay': 'canplay',
    'oncanplaythrough': 'canplaythrough',
    'onchange': 'change',
    'onclick': 'click',
    'onclose': 'close',
    'oncontextlost': 'contextlost',
    'oncontextmenu': 'contextmenu',
    'oncontextrestored': 'contextrestored',
    'oncopy': 'copy',
    'oncuechange': 'cuechange',
    'oncut': 'cut',
    'ondblclick': 'dblclick',
    'ondrag': 'drag',
    'ondragend': 'dragend',
    'ondragenter': 'dragenter',
    'ondragleave': 'dragleave',
    'ondragover': 'dragover',
    'ondragstart': 'dragstart',
    'ondrop': 'drop',
    'ondurationchange': 'durationchange',
    'onemptied': 'emptied',
    'onended': 'ended',
    'onformdata': 'formdata',
    'oninput': 'input',
    'oninvalid': 'invalid',
    'onkeydown': 'keydown',
    'onkeypress': 'keypress',
    'onkeyup': 'keyup',
    'onloadeddata': 'loadeddata',
    'onloadedmetadata': 'loadedmetadata',
    'onloadstart': 'loadstart',
    'onmousedown': 'mousedown',
    'onmouseenter': 'mouseenter',
    'onmouseleave': 'mouseleave',
    'onmousemove': 'mousemove',
    'onmouseout': 'mouseout',
    'onmouseover': 'mouseover',
    'onmouseup': 'mouseup',
    'onpaste': 'paste',
    'onpause': 'pause',
    'onplay': 'play',
    'onplaying': 'playing',
    'onprogress': 'progress',
    'onratechange': 'ratechange',
    'onreset': 'reset',
    'onsecuritypolicyviolation': 'securitypolicyviolation',
    'onseeked': 'seeked',
    'onseeking': 'seeking',
    'onselect': 'select',
    'onslotchange': 'slotchange',
    'onstalled': 'stalled',
    'onsubmit': 'submit',
    'onsuspend': 'suspend',
    'ontimeupdate': 'timeupdate',
    'ontoggle': 'toggle',
    'onvolumechange': 'volumechange',
    'onwaiting': 'waiting',
    'onwebkitanimationend': 'webkitAnimationEnd',
    'onwebkitanimationiteration': 'webkitAnimationIteration',
    'onwebkitanimationstart': 'webkitAnimationStart',
    'onwebkittransitionend': 'webkitTransitionEnd',
    'onwheel': 'wheel',
    'onblur': 'blur',
    'onerror': 'error',
    'onfocus': 'focus',
    'onload': 'load',
    'onresize': 'resize',
    'onscroll': 'scroll',
    'onscrollend': 'scrollend',
    'onafterprint': 'afterprint',
    'onbeforeprint': 'beforeprint',
    'onbeforeunload': 'beforeunload',
    'onhashchange': 'hashchange',
    'onlanguagechange': 'languagechange',
    'onmessage': 'message',
    'onmessageerror': 'messageerror',
    'onoffline': 'offline',
    'ononline': 'online',
    'onpagehide': 'pagehide',
    'onpageshow': 'pageshow',
    'onpopstate': 'popstate',
    'onrejectionhandled': 'rejectionhandled ',
    'onstorage': 'storage',
    'onunhandledrejection': 'unhandledrejection',
    'onunload': 'unload',
    'onreadystatechange': 'readystatechange',
    'onvisibilitychange': 'visibilitychange',

    # custom
    '_oncontentchange': '_contentchange',
}
